<?php

namespace TVn7\DomainBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use TVn7\DomainBundle\Entity\Target;

use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\Context\ExecutionContextInterface;
use Gedmo\Mapping\Annotation as Gedmo;

use Doctrine\Common\Collections\ArrayCollection;

use TVn7\UserBundle\Entity\Membre;

/*
 * TVn7DomainBundle:Alias
 * Cette entité est utilisée par Postfix sur Galileo pour gérer les mails de TVn7.
 *
 * Champs critiques :
 *  * addr
 *  * enabled
 * 
 * Plus d'info ici : https://git.inpt.fr/inp-net/conf-mail/blob/master/tvn7.cf
 */

/**
 * Alias
 *
 * @ORM\Table(name="tdb_domain_alias")
 * @ORM\Entity(repositoryClass="TVn7\DomainBundle\Repository\AliasRepository")
 * @UniqueEntity(fields="addr", message="Cet alias existe déjà")
 * @Gedmo\Loggable(logEntryClass="TVn7\LogBundle\Entity\AppsLog")
 */
class Alias
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="addr", type="string", length=255, unique=true)
     * @Assert\Email(
     *     message = "{{ value }} n'est pas un email valide.",
     *     checkMX = false,
     *     strict = true
     * )
     * @Gedmo\Versioned
     */
    private $addr;

    /**
     * @var bool
     *
     * @ORM\Column(name="enabled", type="boolean", options={"default": 1})
     * @Gedmo\Versioned
     */
    private $enabled = true;

    /**
     * @var bool
     *
     * @ORM\Column(name="locked", type="boolean", options={"default": 0})
     * @Gedmo\Versioned
     */
    private $locked = false;

    /**
     * @var bool
     *
     * @ORM\Column(name="starred", type="boolean", options={"default": 0})
     * @Gedmo\Versioned
     */
    private $starred = false;

    /**
     * @var Array
     * @ORM\OneToMany(targetEntity="TVn7\DomainBundle\Entity\Target", mappedBy="alias", cascade={"all"}, fetch="EAGER")
     * @ORM\OrderBy({"addr" = "ASC"})
     * @Assert\Valid(payload = {"severity" = "warning"})
     */
    private $targets;

    /**
     * @var Membre
     *
     * @ORM\ManyToMany(targetEntity="TVn7\UserBundle\Entity\Membre", inversedBy="aliases", fetch="EAGER")
     * @ORM\JoinTable(name="tdb_domain_alias_owners")
     */
    private $owners;

    /**
     * Constructor
     *
     * @param string $addr
     */
    public function __construct($addr = null)
    {
        $this->targets = new \Doctrine\Common\Collections\ArrayCollection();
        $this->owners = new \Doctrine\Common\Collections\ArrayCollection();
        if($addr != null) $this->addr = $addr;
    }
    
    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set addr
     *
     * @param string $addr
     *
     * @return Alias
     */
    public function setAddr($addr)
    {
        $this->addr = strtolower($addr);

        return $this;
    }

    /**
     * Get addr
     *
     * @return string
     */
    public function getAddr()
    {
        return $this->addr;
    }

    /**
     * Set enabled
     *
     * @param boolean $enabled
     *
     * @return Alias
     */
    public function setEnabled($enabled)
    {
        $this->enabled = $enabled;

        return $this;
    }

    /**
     * Get enabled
     *
     * @return bool
     */
    public function getEnabled()
    {
        return $this->enabled;
    }

    /**
     * Set locked
     *
     * @param boolean $locked
     *
     * @return Alias
     */
    public function setLocked($locked)
    {
        $this->locked = $locked;

        return $this;
    }

    /**
     * Get locked
     *
     * @return bool
     */
    public function getLocked()
    {
        return $this->locked;
    }

    /**
     * Set starred
     *
     * @param boolean $starred
     *
     * @return Alias
     */
    public function setStarred($starred)
    {
        $this->starred = $starred;

        return $this;
    }

    /**
     * Get starred
     *
     * @return bool
     */
    public function getStarred()
    {
        return $this->starred;
    }

    /**
     * Add target
     *
     * @param Target $target
     *
     * @return Alias
     */
    public function addTarget(Target $target)
    {
        $target->setAlias($this);
        $this->targets[] = $target;

        return $this;
    }

    /**
     * Add target
     *
     * @param string $addr
     *
     * @return Alias
     */
    public function addTargetAddr($addr)
    {
        $target = new Target($addr);
        $target->setAlias($this);
        $this->targets[] = $target;

        return $this;
    }

    /**
     * Remove target
     *
     * @param Target $target
     */
    public function removeTarget(Target $target)
    {
        $this->targets->removeElement($target);
    }

    /**
     * Remove all targets
     *
     */
    public function clearTargets()
    {
        $this->targets->clear();

        return $this;
    }

    /**
     * Get targets
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getTargets()
    {
        return $this->targets;
    }

    /**
     * Set targets
     *
     * @return Alias
     */
    public function setTargets(ArrayCollection $values)
    {
        $this->targets = $values;
        return $this;
    }

    public function __toString()
    {
        return $this->getAddr();
    }

    /**
     * @Assert\Callback
     */
    public function isValid(ExecutionContextInterface $context, $payload) {
        
    }

    /**
     *
     * @return array
     */
    public function getTargetsInArray() {
        return $this->getTargets()->map(function($p) {
            return $p->getAddr();
        })->toArray();
    }

    /**
     * Add owner
     *
     * @param \TVn7\UserBundle\Entity\Membre $owner
     *
     * @return Alias
     */
    public function addOwner(\TVn7\UserBundle\Entity\Membre $owner)
    {
        if(!$this->owners->contains($owner)) {
            $this->owners[] = $owner;
        }

        return $this;
    }

    /**
     * Remove owner
     *
     * @param \TVn7\UserBundle\Entity\Membre $owner
     */
    public function removeOwner(\TVn7\UserBundle\Entity\Membre $owner)
    {
        $this->owners->removeElement($owner);
    }

    /**
     * Get owners
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getOwners()
    {
        return $this->owners;
    }
}
